@@ -2,7 +2,7 @@
 #
 # Part of the Rattle package for Data Mining
 #
-# Time-stamp: <2009-08-01 20:43:33 Graham Williams>
+# Time-stamp: <2009-10-03 08:12:04 Graham Williams>
 #
 # Copyright (c) 2009 Togaware Pty Ltd
 #
@@ -24,27 +24,29 @@
 ########################################################################
 # rpart PMML exporter
 #
-# Original: by Togaware
-# 
-# Updated: updated regression tree, 
-#          added ScoreDistribution, missingValueStrategy 
-#          by Zementis, Inc. in June 2008
-#           
-# Updated: updated by Zementis Inc. to add Output element in July 2009
+# Original by Togaware
+# Updated by Zementis 080605 to add ScoreDistribution, missingValueStrategy 
+# Updated by Zementis 090705 to add Output element
+# Updated by Togaware 091003 to add other attributes to Output element
+#
+# To Do
+#   Add the optional ModelStats, ModelExplanation and ModelVerification elements
+
 pmml.rpart <- function(model,
                        model.name="RPart_Model",
                        app.name="Rattle/PMML",
                        description="RPart Decision Tree Model",
                        copyright=NULL,
                        transforms=NULL,
+                       dataset=NULL,
                         ...)
 {
   if (! inherits(model, "rpart")) stop("Not a legitimate rpart object")
   require(XML, quietly=TRUE)
   require(rpart, quietly=TRUE)
 
-  functionName <- "classification"
-  if (model$method != "class") functionName <- "regression"
+  function.name <- "classification"
+  if (model$method != "class") function.name <- "regression"
   
   # Collect the required information.
 
@@ -86,6 +88,10 @@ pmml.rpart <- function(model,
       field$name <- field$name[-unused]
       field$class <- field$class[-unused]
     }
+
+    # 090813 Ensure transforms that remain are made active.
+
+    transforms <- activateTransforms(transforms)
   }
   
   # 081229 Our names and types get out of sync for multiple transforms
@@ -106,11 +112,18 @@ pmml.rpart <- function(model,
   
   for (i in 1:number.of.fields)
   {
-    if (field$class[[field$name[i]]] == "factor")
-      if (field$name[i] == target)
+    # 090829 Zementis Move the test for factor to inside the test for
+    # a target.  This will guarantee that for a non-string target in a
+    # classification tree model, the probability can still be output
+    # for each category.
+
+    # if (field$class[[field$name[i]]] == "factor")
+
+    if (field$name[i] == target)
         field$levels[[field$name[i]]] <- attr(model, "ylevels")
       else
-        field$levels[[field$name[i]]] <- attr(model,"xlevels")[[field$name[i]]]
+        if (field$class[[field$name[i]]] == "factor")
+          field$levels[[field$name[i]]] <- attr(model,"xlevels")[[field$name[i]]]
   }
 
   # 090519 Identify those variables that are not used in the model. We
@@ -139,12 +152,12 @@ pmml.rpart <- function(model,
 
   # PMML -> DataDictionary
 
-  pmml <- append.XMLNode(pmml, pmmlDataDictionary(field))
+  pmml <- append.XMLNode(pmml, pmmlDataDictionary(field, dataset))
 
   # PMML -> TreeModel
 
   the.model <- xmlNode("TreeModel", attrs=c(modelName=model.name,
-                                       functionName=functionName,
+                                       functionName=function.name,
                                        algorithmName="rpart",
                                        splitCharacteristic="binarySplit",
                                        missingValueStrategy="defaultChild"))
@@ -153,11 +166,14 @@ pmml.rpart <- function(model,
   
   the.model <- append.XMLNode(the.model, pmmlMiningSchema(field, target, inactive))
 
-  #########################################
-  #  OUTPUT
-  the.model <- append.XMLNode(the.model, pmmlOutput(field,target))
+  # PMML -> TreeModel -> Output
+  
+  the.model <- append.XMLNode(the.model,
+                              pmmlOutput(field, target,
+                                         switch(function.name,
+                                                classification="categorical",
+                                                regression="continuous")))
 
-  ############################################################################
   # PMML -> TreeModel -> LocalTransformations -> DerivedField -> NormContiuous
 
   if (supportTransformExport(transforms))
@@ -183,10 +199,10 @@ pmml.rpart <- function(model,
   rows <- (1:length(id))[parent.cp > cp]
   parent_ii <- 1
 
-  # Check the functionName.
+  # Check the function.name.
   
   score <- attr(model, "ylevels")[model$frame$yval]
-  if(functionName == "regression") score <- ff$yval[rows]
+  if(function.name == "regression") score <- ff$yval[rows]
 
   # Get the information for the primary predicates
 
